`timescale 1ns/1ns

module test ;
   reg          clk, rstn ;

   initial begin
      rstn      = 0 ;
      #8 rstn   = 1 ;
      forever begin
         clk = 0 ; # 5;
         clk = 1 ; # 5;
      end
   end

   //======================(1)task
   reg  [3:0]   a, b;
   wire [3:0]   co ;
   initial begin
      a         = 0 ;
      b         = 0 ;
      sig_input(4'b1111, 4'b1001, a, b);
      sig_input(4'b0110, 4'b1001, a, b);
      sig_input(4'b1000, 4'b1001, a, b);
   end

   task sig_input ;
      input [3:0]       a ;
      input [3:0]       b ;
      output [3:0]      ao ;
      output [3:0]      bo ;
      @(posedge clk) ;
      ao = a ;
      bo = b ;
   endtask ; // sig_input

   xor_oper         u_xor_oper
     (
      .clk              (clk  ),
      .rstn             (rstn ),
      .a                (a    ),
      .b                (b    ),
      .co               (co   ));

   //===================(2)
   //when no global variables
   task clk_rvs_iner ;
      output    clk_no_rvs ;
      # 5 ;     clk_no_rvs = 0 ;
      # 5 ;     clk_no_rvs = 1 ;
   endtask // clk_rvs_iner
   reg          clk_test1 ;
   always clk_rvs_iner(clk_test1);

   //use task to operate global varialbes
   reg          clk_test2 ;
   task clk_rvs_global ;
      # 5 ;     clk_test2 = 0 ;
      # 5 ;     clk_test2 = 1 ;
   endtask // clk_rvs_iner
   always clk_rvs_global;


   //====================(3)
   //automatic task
   //task test_flag ;
   task automatic test_flag ;
      input [3:0]       cnti ;
      input             en ;
      output [3:0]      cnto ;
      if (en) cnto = cnti ;
   endtask

   reg          en_cnt ;
   reg [3:0]    cnt_temp ;
   initial begin
      en_cnt    = 1 ;
      cnt_temp  = 0 ;
      #45 ;     en_cnt = 0 ;
   end
   always #10 cnt_temp = cnt_temp + 1 ;

   reg [3:0]            cnt1, cnt2 ;
   always @(posedge clk) test_flag(2, en_cnt, cnt1);
   always @(posedge clk) test_flag(cnt_temp, !en_cnt, cnt2);

   //stop
   initial begin
      forever begin
         #100;
         if ($time >= 1000)  $finish ;
      end
   end

endmodule // test
